	org $4000

; two entry-point routines are defined here : initstruct, parabolic_step
        jmp     initstruct
        jmp     parabolic_step

structs	equ	$5000		; bullet structures are located in this page,
				; so we only need an offset to specify one bullet

; each bullet structure consists of the following fields:
pos_x		equ	structs			; x (1 byte)
pos_y		equ	structs+1		; y (1 byte)
x_speed		equ	structs+2		; vx (initial speed) (2 bytes, fractional range -1/2..1/2)
y_speed		equ	structs+4		; vy (initial speed) (2 bytes, fractional range -1/2..1/2)
gravity		equ	structs+6		; g (1 byte, fractional range 0..1/256)
pos_addr	equ	structs+7		; address on screen
pos_bit		equ	structs+9		; bit weight in screen byte ($40,$20,$10,8,4,2,1)
y_delta		equ	structs+10		; dy for internal use (2 bytes, fractional range -1/2..1/2)
x_fract		equ	structs+12		; x fractional part (2 bytes, fractional range -1/2..1/2)
y_fract		equ	structs+14		; y fractional part (2 bytes, fractional range -1/2..1/2)

	
; parabolic_step updates a bullet structure (ie. computes the bullet position reached after a time step)

parabolic_step		; parameter in : reg X = offset to a bullet structure
			; modified registers : A, P
			; results : none, except the bullet structure has been updated

; first, compute dy = g.t.dt + vy.dt
; this was g.(t-dt).dt + vy.dt last time we computed it,
; so we just need to add g.dt.dt to the previous value
; Of course, we aren't crazy, so we choose dt=1
	clc
	lda	y_delta,x
	adc	gravity,x
	sta	y_delta,x
	lda	y_delta+1,x
        adc     #0
        sta     y_delta+1,x
        bvc     compute_x
        lda     #$7F                    ; ok, we limit vertical speed
        sta     y_delta+1,x

; second, compute x = vx.t + x0
; we just need to add dx to the previous value, and dx is vx.dt,
; so this is easy.
; However, for smooth operation, the computation is split
; into a fractional part and an integer part, with just an odd
; thing which is the fractional part is signed.
compute_x
        clc
	lda	x_fract,x
	adc	x_speed,x
	sta	x_fract,x
	lda	x_fract+1,x
	adc	x_speed+1,x
	sta	x_fract+1,x
	bvc	compute_y		; no overflow, so x doesn't change
	bpl	decrement_x
					; we got an overflow and a negative result
					; this means it should have been positive,
					; so we need to increment x
	inc	pos_x,x
					; also, we update a fast pointer to screen location
					; so that users of this routine don't have to
					; translate coordinates to addresses.
					; Alternatively, if you want to use tables to quickly
					; compute screen addresses from x,y coordinates,
					; just suppress the code handling pos_addr and pos_bit
	lsr	pos_bit,x
	bcc	compute_y
        lda     #$20
	sta	pos_bit,x
	inc	pos_addr,x
	bne	compute_y
	inc	pos_addr+1,x
	bne	compute_y

decrement_x
					; here we got an overflow and a positive result
					; so it should have been negative
	dec	pos_x,x

	lda	pos_bit,x
        asl     a
	sta	pos_bit,x
	cmp	#$40
	bne	compute_y
	lda	#1
	sta	pos_bit,x
	lda	pos_addr,x
	bne	*+5
	dec	pos_addr+1,x	
	dec	pos_addr,x

; third, compute y, we just need to add dy to the previous value.
; Once again, computation is split into a fractional part and an integer part
compute_y
	clc
	lda	y_fract,x
	adc	y_delta,x
	sta	y_fract,x
	lda	y_fract+1,x
	adc	y_delta+1,x
	sta	y_fract+1,x
	bvc	end_parabolic		; no overflow, so y doesn't change
	bpl	decrement_y

	inc	pos_y,x
	clc
	lda	pos_addr,x
	adc	#40
	sta	pos_addr,x
	bcc	*+5
	inc	pos_addr+1,x
	jmp	end_parabolic

decrement_y
	dec	pos_y,x
	sec
	lda	pos_addr,x
	sbc	#40
	sta	pos_addr,x
	bcs	*+5
	dec	pos_addr+1,x

end_parabolic
	rts


divider		db	0
div_result	db	0
weight          db      $20,$10,8,4,2,1

; initstruct fills the fields of a bullet structure
;   this mainly computes the initial address of the bullet, given its (x,y) coordinates,
;   and prepares dy, given the initial speed.

initstruct		; parameter in : reg X = offset to a bullet structure
			;   (fields pos_x, pos_y, x_speed, y_speed, gravity must have been set)
			; modified registers : A, P, Y
			; results : none, except the other fields of the bullet structure have been updated
			;   (including the initial address and bit weight fields)
	lda	#0
	sta	x_fract,x
	sta	x_fract+1,x
	sta	y_fract,x
	sta	y_fract+1,x
	sta	div_result
				; dy = g.t.dt + vy.dt, so prepare dy as vy
	lda	y_speed,x
	sta	y_delta,x
	lda	y_speed+1,x
	sta	y_delta+1,x
				; multiply y by 40, and add $A000, this gives the address of the line
	lda	#$05		; we are going to rol this byte five times, thus giving $A0
	sta	pos_addr+1,x
	lda	pos_y,x
        asl     a
	rol	pos_addr+1,x
        asl     a
	rol	pos_addr+1,x
	adc	pos_y,x
	bcc	*+5
	inc	pos_addr+1,x
        asl     a
	rol	pos_addr+1,x
        asl     a
	rol	pos_addr+1,x
        asl     a
	rol	pos_addr+1,x
	sta	pos_addr,x
				; divide x by 6
				; well, this would be much faster with a table, but
				; are we going to spend so much memory ?
	lda	#$C0		; $C0 is 6 shifted 5 times
	sta	divider
	lda	pos_x,x
	ldy	#5
div6loop
	cmp	divider
	bcc	*+5
	sbc	divider
	rol	div_result
	ror	divider
	dey
	bpl	div6loop
				; put the weight of the reminder in pos_bit
	tay
	lda	weight,y
        sta     pos_bit,x
				; and add the result of the division to the address of the line
	clc
	lda	pos_addr,x
	adc	div_result
	sta	pos_addr,x
	bcc	*+5
	inc	pos_addr+1,x
	rts
